﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Media3D;


namespace sphere1
{
    public partial class MainWindow : Window
    {
        private const int maxLongitudes = 100; //maximum
        private const int maxLatitudes = 100; //maximum
        private Point3D[,] position = new Point3D[maxLongitudes + 1, maxLatitudes];
        private Point[,] texture = new Point[maxLongitudes + 1, maxLatitudes];
        private DiffuseMaterial[] frontMaterial = new DiffuseMaterial[maxLatitudes - 1];
        private int longitudes; //actual <= maximum
        private int latitudes; //actual <= maximum
        private int emptySlices = 0; //cuts slices out of the apple
        private int IndexOfFirstGeometryModel3DInModel3DGroup; //= no of lights = 2
        private Matrix3D matrix = Matrix3D.Identity;
        private MatrixTransform3D matrixTransform3D;
        private Quaternion qX = new Quaternion(new Vector3D(1, 0, 0), 1); //rotations around X-axis
        private Quaternion qY = new Quaternion(new Vector3D(0, 1, 0), 1); //rotations around Y-axis
        private Quaternion qZ = new Quaternion(new Vector3D(0, 0, 1), 1); //rotations around Z-axis
        private System.Windows.Threading.DispatcherTimer timer =
        new System.Windows.Threading.DispatcherTimer();


        public MainWindow() //constructor
        {
            InitializeComponent();
            IndexOfFirstGeometryModel3DInModel3DGroup = model3DGroup.Children.Count;
            longitudes = 30;
            latitudes = 30;

            StartAnimation();

            GenerateImageMaterials();
            GenerateSphere();
            GenerateAllCylinders();
            timer.Interval = TimeSpan.FromMilliseconds(50);
            timer.Tick += TimerOnTick;
        }

        private void StartAnimation()
        {
            timer.Stop();
            matrix = Matrix3D.Identity;
            timer.Start();
        }

        private void TimerOnTick(Object sender, EventArgs args)
        {
            matrix.Rotate(qY);
            matrixTransform3D = new MatrixTransform3D(matrix);
            for (int i = IndexOfFirstGeometryModel3DInModel3DGroup;
            i < model3DGroup.Children.Count; i++)
                ((GeometryModel3D)model3DGroup.Children[i]).Transform = matrixTransform3D;


        }



        protected override void OnRenderSizeChanged(SizeChangedInfo sizeInfo)
        {
            try { viewport.Width = viewport.Height = window.ActualWidth; }
            catch { }
        }

        private void GenerateImageMaterials()
        {
            ImageBrush imageBrush;
            double flatThickness = 1.0 / (latitudes - 1);
            double minus = (double)(longitudes - emptySlices);

            for (int lat = 0; lat < latitudes - 1; lat++)
            {
                imageBrush = new ImageBrush((BitmapImage)Resources["earthImage"]);
                imageBrush.Viewbox = new Rect(0, lat * flatThickness, minus / longitudes,
                flatThickness);
                frontMaterial[lat] = new DiffuseMaterial(imageBrush);
            }

        }


        private void GenerateSphere()
        {
            int longitudes = 30;
            int latitudes = 30;
            double latitudeArcusIncrement = Math.PI / (latitudes - 1);
            double longitudeArcusIncrement = 2.0 * Math.PI / longitudes;
            for (int lat = 0; lat < latitudes; lat++)
            {
                double latitudeArcus = lat * latitudeArcusIncrement;
                double radius = Math.Sin(latitudeArcus);
                //if ( lat == latitudes/2 ) radius *= 1.3;
                double y = Math.Cos(latitudeArcus);
                double textureY = (double)lat / (latitudes - 1);
                for (int lon = 0; lon <= longitudes; lon++)
                {
                    double longitudeArcus = lon * longitudeArcusIncrement;
                    position[lon, lat].X = radius * Math.Cos(longitudeArcus);
                    position[lon, lat].Y = y;
                    position[lon, lat].Z = -radius * Math.Sin(longitudeArcus);
                    texture[lon, lat].X = (double)lon / longitudes;
                    texture[lon, lat].Y = textureY;
                }
            }
        }

        private void GenerateAllCylinders()
        { //At first delete all existing flats beginning with the last one
            for (int i = model3DGroup.Children.Count - 1;
            i >= IndexOfFirstGeometryModel3DInModel3DGroup; i--)
                model3DGroup.Children.Remove((GeometryModel3D)model3DGroup.Children[i]);
            for (int lat = 0; lat < latitudes - 1; lat++)
            {
                GeometryModel3D geometryModel3D = new GeometryModel3D();
                geometryModel3D.Geometry = GenerateCylinder(lat);
                geometryModel3D.Material = frontMaterial[lat];
                geometryModel3D.BackMaterial = (DiffuseMaterial)Resources["backMaterial"];
                model3DGroup.Children.Add(geometryModel3D);
            }
        }



        private MeshGeometry3D GenerateCylinder(int lat)
        {
            MeshGeometry3D meshGeometry3D = new MeshGeometry3D();
            for (int lon = 0; lon <= longitudes - emptySlices; lon++)
            //create a zigzag point collection
            {
                Point3D p0 = position[lon, lat]; //on the ceiling
                Point3D p1 = position[lon, lat + 1]; //on the floor
                meshGeometry3D.Positions.Add(p0); //on the ceiling
                meshGeometry3D.Positions.Add(p1); //on the floor
                meshGeometry3D.Normals.Add((Vector3D)p0); //ceiling normal
                meshGeometry3D.Normals.Add((Vector3D)p1); //floor normal
                meshGeometry3D.TextureCoordinates.Add(texture[lon, lat]); //on the ceiling
                meshGeometry3D.TextureCoordinates.Add(texture[lon, lat + 1]); //on the floor
            }

            for (int lon = 1; lon < meshGeometry3D.Positions.Count - 2; lon += 2)
            { //first triangle = left upper part of a rectangle
                meshGeometry3D.TriangleIndices.Add(lon - 1); //left upper point
                meshGeometry3D.TriangleIndices.Add(lon); //left lower point
                meshGeometry3D.TriangleIndices.Add(lon + 1); //right upper point
                //second triangle = right lower part of the rectangle
                meshGeometry3D.TriangleIndices.Add(lon + 1); //right upper point
                meshGeometry3D.TriangleIndices.Add(lon); //left lower point
                meshGeometry3D.TriangleIndices.Add(lon + 2); //right lower point
            }


            return meshGeometry3D;
        }

    }
}
